import ricTracker from './ricTracker';
import frameTracker from './frameTracker';
import waitHiddenTask from './utils/waitHiddenTask';
export default class WorkCycleTracker {
    #workCycleStart = -1;
    startTracking() {
        ricTracker.start();
        frameTracker.start();
    }
    requestStopTracking() {
        ricTracker.stop();
        frameTracker.requestStop();
    }
    canWorkMore(task) {
        const isInputPending = navigator.scheduling?.isInputPending?.() === true;
        return !isInputPending && this.#calculateDeadline(task) - Date.now() > 0;
    }
    async nextWorkCycle(task) {
        if (task.type === 'frame-based') {
            await Promise.race([frameTracker.waitAfterFrame(), waitHiddenTask()]);
        }
        else if (task.type === 'idle-based') {
            if (ricTracker.available) {
                await ricTracker.waitIdleCallback();
            }
            else {
                // todo: use waitHiddenTask() with a timeout
                await frameTracker.waitAfterFrame();
            }
        }
        this.#workCycleStart = Date.now();
    }
    #calculateDeadline(task) {
        if (task.type === 'frame-based') {
            // const timePerFrame = 1000 / fps.guessRefreshRate()
            // const multiplier = timePerFrame / fps.guessRefreshRate()
            // const maxWorkTime = fps.fps() * multiplier
            // return this.#workCycleStart + maxWorkTime
            return this.#workCycleStart + task.workTime;
        }
        else if (task.type === 'idle-based') {
            const idleDeadline = ricTracker.deadline === undefined
                ? Number.MAX_SAFE_INTEGER
                : Date.now() + ricTracker.deadline.timeRemaining();
            return Math.min(this.#workCycleStart + task.workTime, idleDeadline);
        }
        return -1;
    }
}
