import withResolvers from './utils/withResolvers';
import waitNextTask from './utils/waitNextTask';
class FrameTracker {
    #timeoutId;
    #requestAnimationId;
    #deferred;
    constructor() {
        this.#deferred = withResolvers();
    }
    async waitAfterFrame() {
        await this.#deferred.promise;
        await waitNextTask();
    }
    start() {
        clearTimeout(this.#timeoutId);
        this.#timeoutId = undefined;
        this.#loop();
    }
    requestStop() {
        if (this.#timeoutId === undefined) {
            this.#timeoutId = setTimeout(() => {
                this.#timeoutId = undefined;
                cancelAnimationFrame(this.#requestAnimationId);
                this.#requestAnimationId = undefined;
            }, 200);
        }
    }
    #loop() {
        if (this.#requestAnimationId === undefined) {
            this.#requestAnimationId = requestAnimationFrame(() => {
                this.#requestAnimationId = undefined;
                this.#deferred.resolve();
                this.#deferred = withResolvers();
                this.#loop();
            });
        }
    }
}
const frameTracker = new FrameTracker();
export default frameTracker;
